package cn.com.qmth.examcloud.core.reports.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.persistence.criteria.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.reports.api.bean.ProjectInfoBean;
import cn.com.qmth.examcloud.core.reports.api.request.GetProjectInfoBeanReq;
import cn.com.qmth.examcloud.core.reports.api.request.UpdateProjectCourseOrgCountReq;
import cn.com.qmth.examcloud.core.reports.api.request.UpdateProjectStatusReq;
import cn.com.qmth.examcloud.core.reports.base.bean.AddProjectReq;
import cn.com.qmth.examcloud.core.reports.base.bean.ProjectBean;
import cn.com.qmth.examcloud.core.reports.base.bean.QueryProjectPageReq;
import cn.com.qmth.examcloud.core.reports.base.bean.UpdateProjectReq;
import cn.com.qmth.examcloud.core.reports.base.bean.UpdateProjectScoreReq;
import cn.com.qmth.examcloud.core.reports.base.enums.AnalyseType;
import cn.com.qmth.examcloud.core.reports.base.enums.DataOrigin;
import cn.com.qmth.examcloud.core.reports.base.enums.ReportStatus;
import cn.com.qmth.examcloud.core.reports.dao.ProjectRepo;
import cn.com.qmth.examcloud.core.reports.dao.entity.ProjectEntity;
import cn.com.qmth.examcloud.core.reports.service.ProjectService;
import cn.com.qmth.examcloud.task.api.ReportsComputeCloudService;
import cn.com.qmth.examcloud.task.api.request.AddReportsComputeReq;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;

@Service
public class ProjectServiceImpl implements ProjectService {
	@Autowired
	private ProjectRepo projectRepo;
	@Autowired
	private ReportsComputeCloudService reportsComputeCloudService;
	@Transactional
	@Override
	public Long addProject(AddProjectReq req, Long rootOrgId) {
		ProjectEntity pe = new ProjectEntity();
		BeanUtils.copyProperties(req, pe);
		pe.setRootOrgId(rootOrgId);
		pe.setEnable(true);
		pe.setReportStatus(ReportStatus.NONE);
		pe.setExamIds(StringUtils.join(req.getExamIds().toArray(), ","));
		projectRepo.save(pe);
		return pe.getId();
	}

	@Transactional
	@Override
	public void updateProject(UpdateProjectReq req, Long rootOrgId) {
		ProjectEntity pe = GlobalHelper.getEntity(projectRepo, req.getId(), ProjectEntity.class);
		if (pe == null) {
			throw new StatusException("1000001", "项目不存在");
		}
		if (!rootOrgId.equals(pe.getRootOrgId())) {
			throw new StatusException("1000002", "RootOrgId不一致");
		}
		pe.setName(req.getName());
		pe.setRemarks(req.getRemarks());
		projectRepo.save(pe);
	}

	@Transactional
	@Override
	public void updateEnable(String ids, Boolean enable, Long rootOrgId) {
		List<Long> idList = Arrays.asList(ids.split(",")).stream().map(str -> Long.parseLong(str))
				.collect(Collectors.toList());
		projectRepo.updateEnable(idList, rootOrgId, enable);
	}

	private String toSqlSearchPattern(String column) {
		if (StringUtils.isBlank(column)) {
			return "%";
		} else {
			column = column.trim().replaceAll("\\s", "%");
			column = "%" + column + "%";
			return column;
		}
	}

	@Override
	public PageInfo<ProjectBean> queryPage(QueryProjectPageReq req, Integer pageNo, Integer pageSize, Long rootOrgId) {
		Long id = req.getId();
		String name = req.getName();
		AnalyseType analyseType = req.getAnalyseType();
		DataOrigin dataOrigin = req.getDataOrigin();
		Specification<ProjectEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));

			if (null != id) {
				predicates.add(cb.equal(root.get("id"), id));
			}
			if (null != analyseType) {
				predicates.add(cb.equal(root.get("analyseType"), analyseType));
			}
			if (null != dataOrigin) {
				predicates.add(cb.equal(root.get("dataOrigin"), dataOrigin));
			}
			if (StringUtils.isNotEmpty(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = PageRequest.of(pageNo - 1, pageSize, new Sort(Direction.DESC, "id"));

		Page<ProjectEntity> projects = projectRepo.findAll(specification, pageRequest);

		List<ProjectBean> ret = Lists.newArrayList();

		for (ProjectEntity cur : projects) {
			ProjectBean bean = new ProjectBean();
			BeanUtils.copyProperties(cur, bean);
			bean.setAnalyseTypeName(bean.getAnalyseType().getDesc());
			bean.setDataOriginName(bean.getDataOrigin().getDesc());
			bean.setReportStatusName(bean.getReportStatus().getDesc());
			if (StringUtils.isNotBlank(cur.getExamIds())) {
				List<Long> temList = Arrays.asList(cur.getExamIds().split(",")).stream().map(str -> Long.parseLong(str))
						.collect(Collectors.toList());
				bean.setExamIds(temList);
			}
			if (StringUtils.isNotBlank(cur.getPartitionDetails())) {
				List<Double> temList = Arrays.asList(cur.getPartitionDetails().split(",")).stream()
						.map(str -> Double.parseDouble(str)).collect(Collectors.toList());
				bean.setPartitionDetails(temList);
			}
			ret.add(bean);
		}
		return new PageInfo<ProjectBean>(projects, ret);
	}

	@Transactional
	@Override
	public void updateProjectStatus(UpdateProjectStatusReq req) {
		Long id = req.getProjectId();
		ProjectEntity pe = GlobalHelper.getEntity(projectRepo, id, ProjectEntity.class);
		if (pe == null) {
			throw new StatusException("1000003", "项目不存在");
		}
		if(req.getStatus()==2) {
			pe.setReportStatus(ReportStatus.COMPUTING);
		}else if(req.getStatus()==3) {
			pe.setReportStatus(ReportStatus.SUCCESS);
		}else if(req.getStatus()==4) {
			pe.setReportStatus(ReportStatus.FAIL);
		}else if(req.getStatus()==5) {
			pe.setReportStatus(ReportStatus.STOP);
		}else {
			throw new StatusException("1000004", "无效的Status");
		}
		projectRepo.save(pe);
	}

	@Override
	public ProjectInfoBean getProjectInfoBean(GetProjectInfoBeanReq req) {
		if (req.getProjectId() == null) {
			throw new StatusException("1000005", "项目id不能为空");
		}
		ProjectEntity pe = GlobalHelper.getEntity(projectRepo, req.getProjectId(), ProjectEntity.class);
		if (pe == null) {
			throw new StatusException("1000006", "项目不存在");
		}
		ProjectInfoBean bean=new ProjectInfoBean();
		BeanUtils.copyProperties(pe, bean);
		bean.setAnalyseType(pe.getAnalyseType().name());
		bean.setDataOrigin(pe.getDataOrigin().name());
		bean.setReportStatus(pe.getReportStatus().name());
		List<Double> partitionDetails = Arrays.asList(pe.getPartitionDetails().split(",")).stream().map(str -> Double.parseDouble(str))
				.collect(Collectors.toList());
		bean.setPartitionDetails(partitionDetails);
		return bean;
	}

	@Transactional
	@Override
	public void addReportsCompute(Long id, Long rootOrgId) {
		ProjectEntity pe = GlobalHelper.getEntity(projectRepo,id, ProjectEntity.class);
		if (pe == null) {
			throw new StatusException("1000021", "项目不存在");
		}
		if (!rootOrgId.equals(pe.getRootOrgId())) {
			throw new StatusException("1000022", "非法操作");
		}
		if(!pe.getEnable()) {
			throw new StatusException("1000023", "项目已禁用");
		}
		if(pe.getReportStatus().equals(ReportStatus.WAITCOMPUTE)) {
			throw new StatusException("1000024", "项目已处于待计算状态");
		}
		if(pe.getReportStatus().equals(ReportStatus.COMPUTING)) {
			throw new StatusException("1000025", "项目已处于计算状态");
		}
		pe.setReportStatus(ReportStatus.WAITCOMPUTE);
		projectRepo.saveAndFlush(pe);
		AddReportsComputeReq req=new AddReportsComputeReq();
		req.setProjectId(id);
		req.setRootOrgId(rootOrgId);
		reportsComputeCloudService.addReportsCompute(req);
	}
	@Transactional
	@Override
	public void updateScore(UpdateProjectScoreReq req,Long rootOrgId) {
		Long id=req.getId();
		ProjectEntity pe = GlobalHelper.getEntity(projectRepo,id, ProjectEntity.class);
		if (pe == null) {
			throw new StatusException("1000031", "项目不存在");
		}
		if (!rootOrgId.equals(pe.getRootOrgId())) {
			throw new StatusException("1000032", "非法操作");
		}
		if(!pe.getEnable()) {
			throw new StatusException("1000033", "项目已禁用");
		}
		if(pe.getReportStatus().equals(ReportStatus.WAITCOMPUTE)) {
			throw new StatusException("1000034", "项目已处于待计算状态");
		}
		if(pe.getReportStatus().equals(ReportStatus.COMPUTING)) {
			throw new StatusException("1000035", "项目已处于计算状态");
		}
		Double passScore=req.getPassScore();
		Double totalScore=req.getTotalScore();
		if(passScore<=0) {
			throw new StatusException("1000036", "及格分数必须大于0");
		}
		if(totalScore<=0) {
			throw new StatusException("1000037", "满分分数必须大于0");
		}
		if(passScore>totalScore) {
			throw new StatusException("1000038", "及格分数不能大于满分分数");
		}
		List<Double> scores=req.getScores();
		Set<Double> set=new HashSet<Double>();
		for(Double d:scores) {
			set.add(d);
		}
		if(set.size()!=scores.size()) {
			throw new StatusException("1000039", "分段分数不能相同");
		}
		Collections.sort(scores);
		if(scores.get(0)<=0) {
			throw new StatusException("1000040", "第一个分段分数必须大于0");
		}
		if(scores.get(scores.size()-1)>=totalScore) {
			throw new StatusException("1000041", "最后一个分段分数必须小于满分分数");
		}
		pe.setPassScore(passScore);
		pe.setTotalScore(totalScore);
		pe.setPartitionCount(scores.size());
		pe.setPartitionDetails(StringUtils.join(scores.toArray(), ","));
		pe.setReportStatus(ReportStatus.WAITCOMPUTE);
		projectRepo.saveAndFlush(pe);
		AddReportsComputeReq areq=new AddReportsComputeReq();
		areq.setProjectId(id);
		areq.setRootOrgId(rootOrgId);
		reportsComputeCloudService.addReportsCompute(areq);
	}
	@Override
	public ProjectEntity findProjectById(Long projectId) {
		ProjectEntity pe = GlobalHelper.getEntity(projectRepo, projectId, ProjectEntity.class);
		return pe;
	}

	@Override
	public void updateProjectCourseOrgCount(UpdateProjectCourseOrgCountReq req) {
		projectRepo.updateProjectCourseOrgCount(req.getProjectId(), req.getCourseCount(), req.getOrgCount());
	}
}
